@@ -296,7 +296,7 @@
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 1377 "parser.y"
+#line 1440 "parser.y"
 {
   char  *id;
   List  *bases;
@@ -325,7 +325,7 @@ typedef union YYSTYPE
   } decl;
   Parm         *tparms;
   struct {
-    String     *op;
+    String     *method;
     Hash       *kwargs;
   } tmap;
   struct {
