@@ -7,21 +7,52 @@
  * Useful functions for emitting various pieces of code.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_emit_cxx[] = "$Id: emit.cxx 10003 2007-10-17 21:42:11Z wsfulton $";
+char cvsroot_emit_cxx[] = "$Id: emit.cxx 11471 2009-07-29 20:52:29Z wsfulton $";
 
 #include "swigmod.h"
 
 /* -----------------------------------------------------------------------------
- * emit_args()
+ * emit_return_variable()
  *
- * Creates a list of variable declarations for both the return value
- * and function parameters.
- *
- * The return value is always called result and arguments arg0, arg1, arg2, etc...
- * Returns the number of parameters associated with a function.
+ * Emits a variable declaration for a function return value.
+ * The variable name is always called result.
+ * n => Node of the method being wrapped
+ * rt => the return type
+ * f => the wrapper to generate code into
  * ----------------------------------------------------------------------------- */
 
-void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
+void emit_return_variable(Node *n, SwigType *rt, Wrapper *f) {
+
+  if (!GetFlag(n, "tmap:out:optimal")) {
+    if (rt && (SwigType_type(rt) != T_VOID)) {
+      SwigType *vt = cplus_value_type(rt);
+      SwigType *tt = vt ? vt : rt;
+      SwigType *lt = SwigType_ltype(tt);
+      String *lstr = SwigType_str(lt, "result");
+      if (SwigType_ispointer(lt)) {
+        Wrapper_add_localv(f, "result", lstr, "= 0", NULL);
+      } else {
+        Wrapper_add_local(f, "result", lstr);
+      }
+      if (vt) {
+        Delete(vt);
+      }
+      Delete(lt);
+      Delete(lstr);
+    }
+  }
+}
+
+/* -----------------------------------------------------------------------------
+ * emit_parameter_variables()
+ *
+ * Emits a list of variable declarations for function parameters.
+ * The variable names are always called arg1, arg2, etc...
+ * l => the parameter list
+ * f => the wrapper to generate code into
+ * ----------------------------------------------------------------------------- */
+
+void emit_parameter_variables(ParmList *l, Wrapper *f) {
 
   Parm *p;
   String *tm;
@@ -29,24 +60,6 @@ void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
   /* Emit function arguments */
   Swig_cargs(f, l);
 
-  /* Handle return type */
-  if (rt && (SwigType_type(rt) != T_VOID)) {
-    SwigType *vt = cplus_value_type(rt);
-    SwigType *tt = vt ? vt : rt;
-    SwigType *lt = SwigType_ltype(tt);
-    String *lstr = SwigType_str(lt, "result");
-    if (SwigType_ispointer(lt)) {
-      Wrapper_add_localv(f, "result", lstr, "= 0", NULL);
-    } else {
-      Wrapper_add_local(f, "result", lstr);
-    }
-    if (vt) {
-      Delete(vt);
-    }
-    Delete(lt);
-    Delete(lstr);
-  }
-
   /* Attach typemaps to parameters */
   /*  Swig_typemap_attach_parms("ignore",l,f); */
 
@@ -78,7 +91,6 @@ void emit_args(SwigType *rt, ParmList *l, Wrapper *f) {
       p = nextSibling(p);
     }
   }
-  return;
 }
 
 /* -----------------------------------------------------------------------------
@@ -188,7 +200,7 @@ void emit_attach_parmmaps(ParmList *l, Wrapper *f) {
  * emit_num_arguments()
  *
  * Calculate the total number of arguments.   This function is safe for use
- * with multi-valued typemaps which may change the number of arguments in
+ * with multi-argument typemaps which may change the number of arguments in
  * strange ways.
  * ----------------------------------------------------------------------------- */
 
@@ -220,7 +232,7 @@ int emit_num_arguments(ParmList *parms) {
  * emit_num_required()
  *
  * Computes the number of required arguments.  This function is safe for
- * use with multi-valued typemaps and knows how to skip over everything
+ * use with multi-argument typemaps and knows how to skip over everything
  * properly. Note that parameters with default values are counted unless
  * the compact default args option is on.
  * ----------------------------------------------------------------------------- */
@@ -332,30 +344,57 @@ static void replace_contract_args(Parm *cp, Parm *rp, String *s) {
 #endif
 
 /* -----------------------------------------------------------------------------
- * int emit_action()
+ * int emit_action_code()
  *
- * Emits action code for a wrapper and checks for exception handling
+ * Emits action code for a wrapper. Adds in exception handling code (%exception).
+ * eaction -> the action code to emit
+ * wrappercode -> the emitted code (output)
  * ----------------------------------------------------------------------------- */
-int emit_action_code(Node *n, Wrapper *f, String *eaction) {
-  /* Look for except feature */
+int emit_action_code(Node *n, String *wrappercode, String *eaction) {
+  assert(Getattr(n, "wrap:name"));
+
+  /* Look for except feature (%exception) */
   String *tm = GetFlagAttr(n, "feature:except");
   if (tm)
     tm = Copy(tm);
   if ((tm) && Len(tm) && (Strcmp(tm, "1") != 0)) {
-    Replaceall(tm, "$name", Getattr(n, "name"));
-    Replaceall(tm, "$symname", Getattr(n, "sym:name"));
-    Replaceall(tm, "$function", eaction);
-    Replaceall(tm, "$action", eaction);
-    Printv(f->code, tm, "\n", NIL);
+    if (Strstr(tm, "$")) {
+      Replaceall(tm, "$name", Getattr(n, "name"));
+      Replaceall(tm, "$symname", Getattr(n, "sym:name"));
+      Replaceall(tm, "$function", eaction); // deprecated
+      Replaceall(tm, "$action", eaction);
+      Replaceall(tm, "$wrapname", Getattr(n, "wrap:name"));
+      String *overloaded = Getattr(n, "sym:overloaded");
+      Replaceall(tm, "$overname", overloaded ? Char(Getattr(n, "sym:overname")) : "");
+
+      if (Strstr(tm, "$decl")) {
+        String *decl = Swig_name_decl(n);
+        Replaceall(tm, "$decl", decl);
+        Delete(decl);
+      }
+      if (Strstr(tm, "$fulldecl")) {
+        String *fulldecl = Swig_name_fulldecl(n);
+        Replaceall(tm, "$fulldecl", fulldecl);
+        Delete(fulldecl);
+      }
+    }
+    Printv(wrappercode, tm, "\n", NIL);
     Delete(tm);
     return 1;
   } else {
-    Printv(f->code, eaction, "\n", NIL);
+    Printv(wrappercode, eaction, "\n", NIL);
     return 0;
   }
 }
 
-void emit_action(Node *n, Wrapper *f) {
+/* -----------------------------------------------------------------------------
+ * int emit_action()
+ *
+ * Emits the call to the wrapped function. 
+ * Adds in exception specification exception handling and %exception code.
+ * ----------------------------------------------------------------------------- */
+String *emit_action(Node *n) {
+  String *actioncode = NewStringEmpty();
   String *tm;
   String *action;
   String *wrap;
@@ -364,11 +403,10 @@ void emit_action(Node *n, Wrapper *f) {
 
   /* Look for fragments */
   {
-    String *f;
-    f = Getattr(n, "feature:fragment");
-    if (f) {
+    String *fragment = Getattr(n, "feature:fragment");
+    if (fragment) {
       char *c, *tok;
-      String *t = Copy(f);
+      String *t = Copy(fragment);
       c = Char(t);
       tok = strtok(c, ",");
       while (tok) {
@@ -392,26 +430,13 @@ void emit_action(Node *n, Wrapper *f) {
     }
     Setattr(n, "wrap:code:done", f_code);
   }
+
   action = Getattr(n, "feature:action");
   if (!action)
     action = Getattr(n, "wrap:action");
   assert(action != 0);
 
-  if (!is_public(n) && (is_member_director(n) || GetFlag(n, "explicitcall"))) {
-    /* In order to call protected virtual director methods from the target language, we need
-     * to add an extra dynamic_cast to call the public C++ wrapper in the director class. */
-    Node *parent = Getattr(n, "parentNode");
-    String *symname = Getattr(parent, "sym:name");
-    String *dirname = NewStringf("SwigDirector_%s", symname);
-    String *dirdecl = NewStringf("%s *darg = 0", dirname);
-    Wrapper_add_local(f, "darg", dirdecl);
-    Printf(f->code, "darg = dynamic_cast<%s *>(arg1);\n", dirname);
-    Delete(dirname);
-    Delete(dirdecl);
-  }
-
   /* Get the return type */
-
   rt = Getattr(n, "type");
 
   /* Emit contract code (if any) */
@@ -419,7 +444,7 @@ void emit_action(Node *n, Wrapper *f) {
     /* Preassertion */
     tm = Getattr(n, "contract:preassert");
     if (Len(tm)) {
-      Printv(f->code, tm, "\n", NIL);
+      Printv(actioncode, tm, "\n", NIL);
     }
   }
   /* Exception handling code */
@@ -439,7 +464,7 @@ void emit_action(Node *n, Wrapper *f) {
     int unknown_catch = 0;
     Printf(eaction, "}\n");
     for (Parm *ep = catchlist; ep; ep = nextSibling(ep)) {
-      String *em = Swig_typemap_lookup_new("throws", ep, "_e", 0);
+      String *em = Swig_typemap_lookup("throws", ep, "_e", 0);
       if (em) {
 	SwigType *et = Getattr(ep, "type");
 	SwigType *etr = SwigType_typedef_resolve_all(et);
@@ -463,14 +488,14 @@ void emit_action(Node *n, Wrapper *f) {
   }
 
   /* Look for except typemap (Deprecated) */
-  tm = Swig_typemap_lookup_new("except", n, "result", 0);
+  tm = Swig_typemap_lookup("except", n, "result", 0);
   if (tm) {
     Setattr(n, "feature:except", tm);
     tm = 0;
   }
 
   /* emit the except feature code */
-  emit_action_code(n, f, eaction);
+  emit_action_code(n, actioncode, eaction);
 
   Delete(eaction);
 
@@ -479,8 +504,9 @@ void emit_action(Node *n, Wrapper *f) {
     /* Postassertion */
     tm = Getattr(n, "contract:postassert");
     if (Len(tm)) {
-      Printv(f->code, tm, "\n", NIL);
+      Printv(actioncode, tm, "\n", NIL);
     }
   }
 
+  return actioncode;
 }
