@@ -7,7 +7,7 @@
  * Guile language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_guile_cxx[] = "$Id: guile.cxx 10562 2008-06-23 21:14:53Z wsfulton $";
+char cvsroot_guile_cxx[] = "$Id: guile.cxx 11133 2009-02-20 07:52:24Z wsfulton $";
 
 #include "swigmod.h"
 
@@ -47,6 +47,7 @@ Guile Options (available with -guile)\n\
      -exportprimitive        - Add the (export ...) code from scmstub into the\n\
                                GOOPS file.\n";
 
+static File *f_begin = 0;
 static File *f_runtime = 0;
 static File *f_header = 0;
 static File *f_wrappers = 0;
@@ -174,7 +175,7 @@ public:
 	  }
 	} else if (strcmp(argv[i], "-procdoc") == 0) {
 	  if (argv[i + 1]) {
-	    procdoc = NewFile(argv[i + 1], (char *) "w");
+	    procdoc = NewFile(argv[i + 1], "w", SWIG_output_files());
 	    if (!procdoc) {
 	      FileErrorDisplay(argv[i + 1]);
 	      SWIG_exit(EXIT_FAILURE);
@@ -299,11 +300,12 @@ public:
     /* Initialize all of the output files */
     String *outfile = Getattr(n, "outfile");
 
-    f_runtime = NewFile(outfile, "w");
-    if (!f_runtime) {
+    f_begin = NewFile(outfile, "w", SWIG_output_files());
+    if (!f_begin) {
       FileErrorDisplay(outfile);
       SWIG_exit(EXIT_FAILURE);
     }
+    f_runtime = NewString("");
     f_init = NewString("");
     f_header = NewString("");
     f_wrappers = NewString("");
@@ -311,6 +313,7 @@ public:
     /* Register file targets with the SWIG file handler */
     Swig_register_filebyname("header", f_header);
     Swig_register_filebyname("wrapper", f_wrappers);
+    Swig_register_filebyname("begin", f_begin);
     Swig_register_filebyname("runtime", f_runtime);
     Swig_register_filebyname("init", f_init);
 
@@ -322,10 +325,10 @@ public:
     goopscode = NewString("");
     goopsexport = NewString("");
 
-    Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
-    Swig_banner(f_runtime);
+    Swig_banner(f_begin);
 
-    Printf(f_runtime, "/* Implementation : GUILE */\n\n");
+    Printf(f_runtime, "\n");
+    Printf(f_runtime, "#define SWIGGUILE\n");
 
     if (!use_scm_interface) {
       if (SwigRuntime == 1)
@@ -358,6 +361,8 @@ public:
       Printf(f_runtime, "\n}\n");
     }
 
+    Printf(f_runtime, "\n");
+
     Language::top(n);
 
     /* Close module */
@@ -392,14 +397,16 @@ public:
     Delete(goopstext);
 
     /* Close all of the files */
-    Dump(f_header, f_runtime);
-    Dump(f_wrappers, f_runtime);
-    Wrapper_pretty_print(f_init, f_runtime);
+    Dump(f_runtime, f_begin);
+    Dump(f_header, f_begin);
+    Dump(f_wrappers, f_begin);
+    Wrapper_pretty_print(f_init, f_begin);
     Delete(f_header);
     Delete(f_wrappers);
     Delete(f_init);
-    Close(f_runtime);
+    Close(f_begin);
     Delete(f_runtime);
+    Delete(f_begin);
     return SWIG_OK;
   }
 
@@ -499,15 +506,15 @@ public:
 				 SWIG_output_directory(),
 				 primitive_name);
       Delete(primitive_name);
-      File *scmstubfile = NewFile(fname, (char *) "w");
+      File *scmstubfile = NewFile(fname, "w", SWIG_output_files());
       if (!scmstubfile) {
 	FileErrorDisplay(fname);
 	SWIG_exit(EXIT_FAILURE);
       }
       Delete(fname);
 
-      Printf(scmstubfile, ";;; -*- buffer-read-only: t -*- vi: set ro: */\n");
-      Printf(scmstubfile, ";;; Automatically generated by SWIG; do not edit.\n\n");
+      Swig_banner_target_lang(scmstubfile, ";;;");
+      Printf(scmstubfile, "\n");
       if (linkage == GUILE_LSTYLE_SIMPLE || linkage == GUILE_LSTYLE_PASSIVE)
 	Printf(scmstubfile, "(define-module (%s))\n\n", mod);
       Delete(mod);
@@ -530,14 +537,14 @@ public:
 
       String *fname = NewStringf("%s%s.scm", SWIG_output_directory(),
 				 module_name);
-      File *goopsfile = NewFile(fname, (char *) "w");
+      File *goopsfile = NewFile(fname, "w", SWIG_output_files());
       if (!goopsfile) {
 	FileErrorDisplay(fname);
 	SWIG_exit(EXIT_FAILURE);
       }
       Delete(fname);
-      Printf(goopsfile, ";;; -*- buffer-read-only: t -*- vi: set ro: */\n");
-      Printf(goopsfile, ";;; Automatically generated by SWIG; do not edit.\n\n");
+      Swig_banner_target_lang(goopsfile, ";;;");
+      Printf(goopsfile, "\n");
       Printf(goopsfile, "(define-module (%s))\n", mod);
       Printf(goopsfile, "%s\n", goopstext);
       Printf(goopsfile, "(use-modules (oop goops) (Swig common))\n");
@@ -1653,7 +1660,7 @@ public:
    * If class_name = "" that means the mapping is for a function or
    * variable not attached to any class.
    * ------------------------------------------------------------ */
-  String *goopsNameMapping(String *name, String_or_char *class_name) {
+  String *goopsNameMapping(String *name, const_String_or_char_ptr class_name) {
     String *n = NewString("");
 
     if (Strcmp(class_name, "") == 0) {
