@@ -10,11 +10,14 @@
  * to easily construct yacc-compatible scanners.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_scanner_c[] = "$Id: scanner.c 9688 2007-04-21 16:00:02Z olly $";
+char cvsroot_scanner_c[] = "$Id: scanner.c 11470 2009-07-29 20:50:39Z wsfulton $";
 
 #include "swig.h"
 #include <ctype.h>
 
+extern String *cparse_file;
+extern int     cparse_start_line;
+
 struct Scanner {
   String *text;			/* Current token value */
   List   *scanobjs;		/* Objects being scanned */
@@ -36,7 +39,7 @@ struct Scanner {
  * Create a new scanner object
  * ----------------------------------------------------------------------------- */
 
-Scanner *NewScanner() {
+Scanner *NewScanner(void) {
   Scanner *s;
   s = (Scanner *) malloc(sizeof(Scanner));
   s->line = 1;
@@ -44,7 +47,7 @@ Scanner *NewScanner() {
   s->nexttoken = -1;
   s->start_line = 1;
   s->yylen = 0;
-  s->idstart = "";
+  s->idstart = NULL;
   s->scanobjs = NewList();
   s->text = NewStringEmpty();
   s->str = 0;
@@ -66,6 +69,7 @@ void DelScanner(Scanner * s) {
   Delete(s->file);
   Delete(s->error);
   Delete(s->str);
+  free(s->idstart);
   free(s);
 }
 
@@ -81,6 +85,7 @@ void Scanner_clear(Scanner * s) {
   Clear(s->text);
   Clear(s->scanobjs);
   Delete(s->error);
+  s->str = 0;
   s->error = 0;
   s->line = 1;
   s->nexttoken = -1;
@@ -114,11 +119,11 @@ void Scanner_push(Scanner * s, String *txt) {
  * call to Scanner_token().
  * ----------------------------------------------------------------------------- */
 
-void Scanner_pushtoken(Scanner * s, int nt, const String_or_char *val) {
+void Scanner_pushtoken(Scanner * s, int nt, const_String_or_char_ptr val) {
   assert(s);
   assert((nt >= 0) && (nt < SWIG_MAXTOKENS));
   s->nexttoken = nt;
-  if (val != s->text) {
+  if ( Char(val) != Char(s->text) ) {
     Clear(s->text);
     Append(s->text,val);
   }
@@ -170,7 +175,8 @@ int Scanner_start_line(Scanner * s) {
  * Change the set of additional characters that can be used to start an identifier.
  * ----------------------------------------------------------------------------- */
 
-void Scanner_idstart(Scanner * s, char *id) {
+void Scanner_idstart(Scanner * s, const char *id) {
+  free(s->idstart);
   s->idstart = Swig_copy_string(id);
 }
 
@@ -198,7 +204,7 @@ static char nextchar(Scanner * s) {
   if ((nc == '\n') && (!s->freeze_line)) 
     s->line++;
   Putc(nc,s->text);
-  return nc;
+  return (char)nc;
 }
 
 /* -----------------------------------------------------------------------------
@@ -207,7 +213,7 @@ static char nextchar(Scanner * s) {
  * Sets error information on the scanner.
  * ----------------------------------------------------------------------------- */
 
-static void set_error(Scanner *s, int line, String_or_char *msg) {
+static void set_error(Scanner *s, int line, const_String_or_char_ptr msg) {
   s->error_line = line;
   s->error = NewString(msg);
 }
@@ -347,7 +353,7 @@ static void get_escape(Scanner *s) {
       } else {
 	char tmp[3];
 	tmp[0] = '\\';
-	tmp[1] = c;
+	tmp[1] = (char)c;
 	tmp[2] = 0;
 	Delitem(s->text, DOH_END);
 	Append(s->text, tmp);
@@ -421,7 +427,8 @@ static int look(Scanner * s) {
 
       /* Look for possible identifiers */
 
-      else if ((isalpha(c)) || (c == '_') || (strchr(s->idstart, c)))
+      else if ((isalpha(c)) || (c == '_') ||
+	       (s->idstart && strchr(s->idstart, c)))
 	state = 7;
 
       /* Look for single character symbols */
@@ -533,7 +540,7 @@ static int look(Scanner * s) {
       break;
     case 10:			/* C++ style comment */
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line,"Unterminated comment");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated comment\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '\n') {
@@ -545,7 +552,7 @@ static int look(Scanner * s) {
       break;
     case 11:			/* C style comment block */
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line,"Unterminated comment");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated comment\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '*') {
@@ -556,7 +563,7 @@ static int look(Scanner * s) {
       break;
     case 12:			/* Still in C style comment */
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line,"Unterminated comment");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated comment\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '*') {
@@ -570,7 +577,7 @@ static int look(Scanner * s) {
 
     case 2:			/* Processing a string */
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line, "Unterminated string");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated string\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '\"') {
@@ -640,7 +647,8 @@ static int look(Scanner * s) {
 	Setline(s->text, Getline(s->str));
 	Setfile(s->text, Getfile(s->str));
 	s->start_line = s->line;
-      } else if (strchr(s->idstart, '%') && ((isalpha(c)) || (c == '_'))) {
+      } else if (s->idstart && strchr(s->idstart, '%') &&
+	         ((isalpha(c)) || (c == '_'))) {
 	state = 7;
       } else if (c == '=') {
 	return SWIG_TOKEN_MODEQUAL;
@@ -652,7 +660,7 @@ static int look(Scanner * s) {
 
     case 40:			/* Process an include block */
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line,"Unterminated code block");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated block\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '%')
@@ -929,7 +937,7 @@ static int look(Scanner * s) {
       /* A character constant */
     case 9:
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line,"Unterminated character constant");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated character constant\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '\'') {
@@ -1044,7 +1052,7 @@ static int look(Scanner * s) {
       /* Reverse string */
     case 900:
       if ((c = nextchar(s)) == 0) {
-	set_error(s,s->start_line,"Unterminated character constant");
+	Swig_error(cparse_file, cparse_start_line, "Unterminated character constant\n");
 	return SWIG_TOKEN_ERROR;
       }
       if (c == '`') {
